function [vm,vmraw]=VMSampMNRB(x,y,D,z,pzt,xb,cut,PScoreMod,Test,dn,b,c,list,NonRedList,dx,Exclude);
%This procedure returns the Khmaladze transform of the Unconditional
%version of the Multinomial Conditional Independence test
%assume that pzt contains the estimated probabilities with the highest
%category stored in pzt(:,end)

%This version of the code eliminates the last choice category form D and
%pzt to avoid redundancy

dimy=size(y,2);

n=size(y,1);
nc=size(cut,1)+1;
switch Test
    case 'Unconditional'
        k=size(y,2);
    case 'Conditional'
        k=size(y,2)+size(z,2);
end
a=n^(-1/(2+k)); %Higher order kernel bandwidth
d=size(z,2);
switch Test
    case 'Unconditional'
          V=[y];  
          k=size(V,2);
    case 'Conditional'
          V=[y,z(:,2:end)];
          k=size(V,2);
end
sv=sort(y(:,1));
xv=sv(floor(x*n),:);
vm=0;
vm1=0;

switch PScoreMod
    case {'OProbit','3valOprobit'} %Ordered Probit
          zt=cut(1)*ones(length(pzt),1)-norminv(pzt(:,1)); %retrieve beta'xt    
          cscut=cumsum(cut);
          cspzt=cumsum(pzt')';
          %compatibility check
          if size(cut,1)==size(cspzt,2)
             for i=2:size(cut)
                 %ModelTest=norminv(cspzt(:,i))+xb-cut(i)*ones(length(zt),1);
                 ModelTest=pzt(:,i)-normcdf(cut(i)-zt)+normcdf(cut(i-1)-zt);
                 if ModelTest>=0.0001*ones(length(zt),1)
                    error('MNVWSamp: ordered probit not compatible with input');
                 end
             end
          end
          cut=[-Inf; cut; inf]; 
          dpzt1=(normpdf(ones(n,1)*cut(2:end)'-zt*ones(1,nc),0,1)-normpdf(ones(n,1)*cut(1:end-1)'-zt*ones(1,nc),0,1));
          dpzt1=dpzt1(:,NonRedList); %remove one coordinate to avoid redundancy
          %construct matrix of derivatives dp/d(beta),dp/d(cut); dpzt is n
          %by (d*dim) where dim is the number of choice categories
          dpzt=[];          
          for i=1:size(dpzt1,2)
              dpzt=[dpzt, (dpzt1(:,i)*ones(1,d)).*z];
              dpzt2=zeros(length(zt),nc-1);
              j=NonRedList(i);
              if j<=size(dpzt1,2)                  
                 dpzt2(:,j)=normpdf(ones(length(zt),1)*cut(j+1)-zt,0,1);
              end
              if j>1
                 dpzt2(:,j-1)=-normpdf(cut(j)*ones(length(zt),1)-zt,0,1);
              end
              dpzt=[dpzt, dpzt2];
          end
   case {'mnlogit','3valmnlogit'} %multinomial Logit
          cut=[-Inf; zeros(nc-1,1); inf]; 
          dpzt1=[];
          xb=[zeros(n,1), xb, zeros(n,1)];
          %compute derivatives of choice probabilities with respect to
          %parameters (see Madalla, p.36)
          for i=1:nc
              if ismember(i,NonRedList)
                 dpzti=[];
                 for j=1:nc-1;
                     if j==i && i<nc
                        dpzti=[dpzti ((pzt(:,i).*(1-pzt(:,i)))*ones(1,d)).*z];                     
                     elseif j~=i && i<nc
                        dpzti=[dpzti ((-(pzt(:,i).*pzt(:,j)))*ones(1,d)).*z];                     
                     elseif i==nc
                        dpzti=[dpzti ((-(pzt(:,i).*pzt(:,j)))*ones(1,d)).*z];                     
                     end %if j
                 end %for j
                 dpzt1=[dpzt1 dpzti];
              end %if ismember
          end %for i          
          dpzt=dpzt1;
end

D=D(:,NonRedList);
pzt=pzt(:,NonRedList);


U=RosenblattBRK(V,a,b,c);
[w,raw]=WvSampleMN(U,D,z,x,pzt,dpzt,dx,Exclude);
[int,a,b]=intersect(list,NonRedList);
w=w(:,b); %only use coordinates for required test
dim=size(w,2);
sv=sort(U(:,1));
xv=sv(floor(x*n),:);

for i=1:n;
       if (U(i,1)<xv)
          ind1=(U<=ones(n,1)*U(i,:));
          ind=ones(n,1);
          for i=1:k;
              ind=ind.*ind1(:,i);
          end
          wv=(w.*(ind*ones(1,dim)))'*ones(n,1)/sqrt(n-length(Exclude));         
          vm=vm+(tr(wv'*wv));     
       end
end
vm=vm/n;
return
